/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4;

import com.thoughtworks.selenium.Selenium;
import org.testatoo.cartridge.html4.evaluator.selenium.SeleniumHtmlEvaluator;
import org.testatoo.cartridge.html4.evaluator.selenium.condition.PageLoaded;
import org.testatoo.core.CartridgeBootstraper;
import org.testatoo.core.ConditionChain;
import org.testatoo.core.Evaluator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

public final class Bootstraper implements CartridgeBootstraper {

    @Override
    public Evaluator init(final Object... params) {
        if (params.length == 1 && params[0] instanceof Selenium) {
            Selenium selenium = (Selenium) params[0];

            ConditionChain conditionChain = ConditionChain.create();
            conditionChain.addCondition(new PageLoaded(selenium));

            SeleniumHtmlEvaluator evaluator = new SeleniumHtmlEvaluator(selenium);
            evaluator.setWaitingCondition(conditionChain);

            return evaluator;
        }
        return null;
    }
}
