/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4;

import org.testatoo.cartridge.html4.component.ListBox;
import org.testatoo.cartridge.html4.element.*;
import org.testatoo.cartridge.html4.element.Object;
import org.testatoo.core.ComponentException;
import org.testatoo.core.EvaluatorHolder;
import org.testatoo.core.component.Component;

import java.util.ArrayList;
import java.util.List;

public final class HtmlComponentFactory {

    private HtmlComponentFactory() {
    }

    public static HtmlEvaluator evaluator() {
        return EvaluatorHolder.get();
    }

    public static Page page() {
        return new Page(evaluator(), findComponent(By.xpath("//html")).id());
    }

    // --------------------- Button ----------------------------------
    public static Button button(String id) {
        return findButton(By.id(id));
    }

    public static Button findButton(By by) {
        return new Button(evaluator(), findComponent(by).id());
    }

    public static List<Button> findButtons(By by) {
        List<Button> buttons = new ArrayList<Button>();
        for (Component component : findComponents(by)) {
            buttons.add(new Button(evaluator(), component.id()));
        }
        return buttons;
    }

    // --------------------- CheckBox ----------------------------------
    public static CheckBox checkbox(String id) {
        return findCheckBox(By.id(id));
    }

    public static CheckBox findCheckBox(By by) {
        return new CheckBox(evaluator(), findComponent(by).id());
    }

    public static List<CheckBox> findCheckBoxes(By by) {
        List<CheckBox> checkboxes = new ArrayList<CheckBox>();
        for (Component component : findComponents(by)) {
            checkboxes.add(new CheckBox(evaluator(), component.id()));
        }
        return checkboxes;
    }

    // --------------------- Radio  ----------------------------------
    public static Radio radio(String id) {
        return findRadio(By.id(id));
    }

    public static Radio findRadio(By by) {
        return new Radio(evaluator(), findComponent(by).id());
    }

    public static List<Radio> findRadios(By by) {
        List<Radio> radios = new ArrayList<Radio>();
        for (Component component : findComponents(by)) {
            radios.add(new Radio(evaluator(), component.id()));
        }
        return radios;
    }

    // --------------------- Hidden ----------------------------------
    public static Hidden hidden(String id) {
        return findHidden(By.id(id));
    }

    public static Hidden findHidden(By by) {
        return new Hidden(evaluator(), findComponent(by).id());
    }

    public static List<Hidden> findHiddens(By by) {
        List<Hidden> hiddens = new ArrayList<Hidden>();
        for (Component component : findComponents(by)) {
            hiddens.add(new Hidden(evaluator(), component.id()));
        }
        return hiddens;
    }

    // --------------------- InputText  ------------------------------
    public static InputText inputText(String id) {
        return findInputText(By.id(id));
    }

    public static InputText findInputText(By by) {
        return new InputText(evaluator(), findComponent(by).id());
    }

    public static List<InputText> findInputTexts(By by) {
        List<InputText> inputtexts = new ArrayList<InputText>();
        for (Component component : findComponents(by)) {
            inputtexts.add(new InputText(evaluator(), component.id()));
        }
        return inputtexts;
    }

    // --------------------- Password  ------------------------------
    public static InputPassword inputPassword(String id) {
        return findInputPassword(By.id(id));
    }

    public static InputPassword findInputPassword(By by) {
        return new InputPassword(evaluator(), findComponent(by).id());
    }

    public static List<InputPassword> findInputPasswords(By by) {
        List<InputPassword> inputpasswords = new ArrayList<InputPassword>();
        for (Component component : findComponents(by)) {
            inputpasswords.add(new InputPassword(evaluator(), component.id()));
        }
        return inputpasswords;
    }

    // --------------------- Img --------------------------------
    public static Img img(String id) {
        return findImg(By.id(id));
    }

    public static Img findImg(By by) {
        return new Img(evaluator(), findComponent(by).id());
    }

    public static List<Img> findImgs(By by) {
        List<Img> images = new ArrayList<Img>();
        for (Component component : findComponents(by)) {
            images.add(new Img(evaluator(), component.id()));
        }
        return images;
    }

    public static Map findMap(By by) {
        return new Map(evaluator(), findComponent(by).id());
    }

    public static List<Map> findMaps(By by) {
        List<Map> maps = new ArrayList<Map>();
        for (Component component : findComponents(by)) {
            maps.add(new Map(evaluator(), component.id()));
        }
        return maps;
    }

    public static Area findArea(By by) {
        return new Area(evaluator(), findComponent(by).id());
    }

    public static List<Area> findAreas(By by) {
        List<Area> areas = new ArrayList<Area>();
        for (Component component : findComponents(by)) {
            areas.add(new Area(evaluator(), component.id()));
        }
        return areas;
    }

    // --------------------- Option group -------------------------
    public static OptionGroup optionGroup(String id) {
        return findOptionGroup(By.id(id));
    }

    public static OptionGroup findOptionGroup(By by) {
        return new OptionGroup(evaluator(), findComponent(by).id());
    }

    public static List<OptionGroup> findOptionGroups(By by) {
        List<OptionGroup> optionGroups = new ArrayList<OptionGroup>();
        for (Component component : findComponents(by)) {
            optionGroups.add(new OptionGroup(evaluator(), component.id()));
        }
        return optionGroups;
    }

    // --------------------- Option  ----------------------------------
    public static Option option(String id) {
        return findOption(By.id(id));
    }

    public static Option findOption(By by) {
        return new Option(evaluator(), findComponent(by).id());
    }

    public static List<Option> findOptions(By by) {
        List<Option> options = new ArrayList<Option>();
        for (Component component : findComponents(by)) {
            options.add(new Option(evaluator(), component.id()));
        }
        return options;
    }

    // --------------------- Select -------------------------------
    public static Select select(String id) {
        return findSelect(By.id(id));
    }

    public static Select findSelect(By by) {
        return new Select(evaluator(), findComponent(by).id());
    }

    public static List<Select> findSelects(By by) {
        List<Select> selects = new ArrayList<Select>();
        for (Component component : findComponents(by)) {
            selects.add(new Select(evaluator(), component.id()));
        }
        return selects;
    }

    // --------------------- DropDown -------------------------------
    public static DropDown dropdown(String id) {
        return findDropDown(By.id(id));
    }

    public static DropDown findDropDown(By by) {
        return new DropDown(evaluator(), findComponent(by).id());
    }

    public static List<DropDown> findDropDowns(By by) {
        List<DropDown> dropdowns = new ArrayList<DropDown>();
        for (Component component : findComponents(by)) {
            dropdowns.add(new DropDown(evaluator(), component.id()));
        }
        return dropdowns;
    }

    // --------------------- Form ----------------------------------
    public static Form form(String id) {
        return findForm(By.id(id));
    }

    public static Form findForm(By by) {
        return new Form(evaluator(), findComponent(by).id());
    }

    public static List<Form> findForms(By by) {
        List<Form> forms = new ArrayList<Form>();
        for (Component component : findComponents(by)) {
            forms.add(new Form(evaluator(), component.id()));
        }
        return forms;
    }

    // --------------------- Links : A, Link, Base -----------------------------------
    public static A a(String id) {
        return findA(By.id(id));
    }

    public static A findA(By by) {
        return new A(evaluator(), findComponent(by).id());
    }

    public static List<A> findAs(By by) {
        List<A> links = new ArrayList<A>();
        for (Component component : findComponents(by)) {
            links.add(new A(evaluator(), component.id()));
        }
        return links;
    }

    public static Link link(String id) {
        return findLink(By.id(id));
    }

    public static Link findLink(By by) {
        return new Link(evaluator(), findComponent(by).id());
    }

    public static List<Link> findLinks(By by) {
        List<Link> links = new ArrayList<Link>();
        for (Component component : findComponents(by)) {
            links.add(new Link(evaluator(), component.id()));
        }
        return links;
    }

    public static Base findBase(By by) {
        return new Base(evaluator(), findComponent(by).id());
    }

    public static List<Base> findBases(By by) {
        List<Base> bases = new ArrayList<Base>();
        for (Component component : findComponents(by)) {
            bases.add(new Base(evaluator(), component.id()));
        }
        return bases;
    }

    // ------------------- Panel / Div -----------------------------
    public static Div div(String id) {
        return findDiv(By.id(id));
    }
    
    public static Div findDiv(By by) {
        return new Div(evaluator(), findComponent(by).id());
    }

    public static List<Div> findDivs(By by) {
        List<Div> divs = new ArrayList<Div>();
        for (Component component : findComponents(by)) {
            divs.add(new Div(evaluator(), component.id()));
        }
        return divs;
    }

    // -------------------- AlertBox --------------------------------
    public static AlertBox alertbox() {
        return new AlertBox(evaluator());
    }

    // -------------------- Table --------------------------------
    public static Table datagrid(String id) {
        return findDatagrid(By.id(id));
    }

    public static Table findDatagrid(By by) {
        return findTable(by);
    }

    public static List<Table> findDatagrids(By by) {
        return findTables(by);
    }

    public static Table table(String id) {
        return findTable(By.id(id));
    }

    public static Table findTable(By by) {
        return new Table(evaluator(), findComponent(by).id());
    }

    public static List<Table> findTables(By by) {
        List<Table> tables = new ArrayList<Table>();
        for (Component component : findComponents(by)) {
            tables.add(new Table(evaluator(), component.id()));
        }
        return tables;
    }

    // ----  Table elements : Caption, THead, TFoot, TBody, ColGroup, Col, TR, TH, TD ------------------
    public static Caption findCaption(By by) {
        return new Caption(evaluator(), findComponent(by).id());
    }

    public static THead findThead(By by) {
        return new THead(evaluator(), findComponent(by).id());
    }

    public static TFoot findTFoot(By by) {
        return new TFoot(evaluator(), findComponent(by).id());
    }

    public static TBody findTBody(By by) {
        return new TBody(evaluator(), findComponent(by).id());
    }

    public static Colgroup findColgroup(By by) {
        return new Colgroup(evaluator(), findComponent(by).id());
    }

    public static Col findCol(By by) {
        return new Col(evaluator(), findComponent(by).id());
    }

    public static Tr findTr(By by) {
        return new Tr(evaluator(), findComponent(by).id());
    }

    public static Th findTh(By by) {
        return new Th(evaluator(), findComponent(by).id());
    }

    public static Td findTd(By by) {
        return new Td(evaluator(), findComponent(by).id());
    }

    // --------------------- File  ------------------------------------
    public static File file(String id) {
        return findFile(By.id(id));
    }

    public static File findFile(By by) {
        return new File(evaluator(), findComponent(by).id());
    }

    public static List<File> findFiles(By by) {
        List<File> files = new ArrayList<File>();
        for (Component component : findComponents(by)) {
            files.add(new File(evaluator(), component.id()));
        }
        return files;
    }

    // --------------------- TextArea  ------------------------------
    public static TextArea textArea(String id) {
        return findTextArea(By.id(id));
    }

    public static TextArea findTextArea(By by) {
        return new TextArea(evaluator(), findComponent(by).id());
    }

    public static List<TextArea> findTextAreas(By by) {
        List<TextArea> textAreas = new ArrayList<TextArea>();
        for (Component component : findComponents(by)) {
            textAreas.add(new TextArea(evaluator(), component.id()));
        }
        return textAreas;
    }

    // --------------------- Label -------------
    public static Label findLabel(By by) {
        return new Label(evaluator(), findComponent(by).id());
    }

    public static List<Label> findLabels(By by) {
        List<Label> labels = new ArrayList<Label>();
        for (Component component : findComponents(by)) {
            labels.add(new Label(evaluator(), component.id()));
        }
        return labels;
    }

    // --------------------- FieldSet -------------
    public static FieldSet findFieldSet(By by) {
        return new FieldSet(evaluator(), findComponent(by).id());
    }

    public static List<FieldSet> findFieldSets(By by) {
        List<FieldSet> fieldsSets = new ArrayList<FieldSet>();
        for (Component component : findComponents(by)) {
            fieldsSets.add(new FieldSet(evaluator(), component.id()));
        }
        return fieldsSets;
    }

    // --------------------- Legend -------------
    public static Legend findLegend(By by) {
        return new Legend(evaluator(), findComponent(by).id());
    }

    public static List<Legend> findLegends(By by) {
        List<Legend> Legends = new ArrayList<Legend>();
        for (Component component : findComponents(by)) {
            Legends.add(new Legend(evaluator(), component.id()));
        }
        return Legends;
    }

    // --------------------- Script -------------
    public static Script findScript(By by) {
        return new Script(evaluator(), findComponent(by).id());
    }

    // --------------------- Object and Param-------------
    public static Object findObject(By by) {
        return new Object(evaluator(), findComponent(by).id());
    }

    public static Param findParam(By by) {
        return new Param(evaluator(), findComponent(by).id());
    }

    //----------------- UL, OL, LI --------------------------
    public static Ul findUl(By by) {
        return new Ul(evaluator(), findComponent(by).id());
    }

    public static Li findLi(By by) {
        return new Li(evaluator(), findComponent(by).id());
    }

    public static Ol findOl(By by) {
        return new Ol(evaluator(), findComponent(by).id());
    }

    //-----------------DL, DT, DD --------------------------------
    public static Dl findDl(By by) {
        return new Dl(evaluator(), findComponent(by).id());
    }

    public static Dt findDt(By by) {
        return new Dt(evaluator(), findComponent(by).id());
    }

    public static Dd findDd(By by) {
        return new Dd(evaluator(), findComponent(by).id());
    }

    // --------------------- FrameSet and Frames -------------
    public static FrameSet findFrameSet(By by) {
        return new FrameSet(evaluator(), findComponent(by).id());
    }

    public static Frame findFrame(By by) {
        return new Frame(evaluator(), findComponent(by).id());
    }

    public static NoFrames findNoFrames(By by) {
        return new NoFrames(evaluator(), findComponent(by).id());
    }

    public static IFrame findIFrame(By by) {
        return new IFrame(evaluator(), findComponent(by).id());
    }

    // --------------------- Text tags : Em, Strong, Cite, Dfn, Code, Samp, Kbd, Var, Abbr, Acronym -------------
    public static Em findEm(By by) {
        return new Em(evaluator(), findComponent(by).id());
    }

    public static Strong findStrong(By by) {
        return new Strong(evaluator(), findComponent(by).id());
    }

    public static Cite findCite(By by) {
        return new Cite(evaluator(), findComponent(by).id());
    }

    public static Dfn findDfn(By by) {
        return new Dfn(evaluator(), findComponent(by).id());
    }

    public static Code findCode(By by) {
        return new Code(evaluator(), findComponent(by).id());
    }

    public static Samp findSamp(By by) {
        return new Samp(evaluator(), findComponent(by).id());
    }

    public static Kbd findKbd(By by) {
        return new Kbd(evaluator(), findComponent(by).id());
    }

    public static Var findVar(By by) {
        return new Var(evaluator(), findComponent(by).id());
    }

    public static Abbr findAbbr(By by) {
        return new Abbr(evaluator(), findComponent(by).id());
    }

    public static Acronym findAcronym(By by) {
        return new Acronym(evaluator(), findComponent(by).id());
    }

    private static Component findComponent(By by) {
        Component component;
        try {
            component = by.findComponent(evaluator());
        } catch (Exception e) {
            throw new ComponentException("Cannot find component defined " + by, e);
        }
        return component;
    }

    private static List<Component> findComponents(By by) {
        List<Component> component = new ArrayList<Component>();
        try {
            component.addAll(by.findComponents(evaluator()));
        } catch (Exception e) {
            throw new ComponentException("Cannot find component defined " + by, e);
        }
        return component;
    }
}
