/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.core;

import org.testatoo.cartridge.html4.By;
import org.testatoo.cartridge.html4.HtmlComponentFactory;
import org.testatoo.cartridge.html4.HtmlEvaluator;
import org.testatoo.cartridge.html4.element.Page;
import org.testatoo.core.component.AlertBox;
import org.testatoo.core.component.Button;
import org.testatoo.core.component.CheckBox;
import org.testatoo.core.component.ComboBox;
import org.testatoo.core.component.Component;
import org.testatoo.core.component.DropDown;
import org.testatoo.core.component.FileDialog;
import org.testatoo.core.component.Image;
import org.testatoo.core.component.Link;
import org.testatoo.core.component.ListBox;
import org.testatoo.core.component.Panel;
import org.testatoo.core.component.PasswordField;
import org.testatoo.core.component.Radio;
import org.testatoo.core.component.TextField;
import org.testatoo.core.component.datagrid.DataGrid;

import java.util.ArrayList;
import java.util.List;

public final class ComponentFactory {

    public static Page page() {
        return HtmlComponentFactory.page();
    }

    public static Component component(String id) {
        return findComponent(By.id(id));
    }

    public static TextField textfield(String id) {
        return HtmlComponentFactory.inputText(id);
    }

    public static ListBox listbox(String id) {
        return findListBox(By.id(id));
    }

    public static PasswordField passwordfield(String id) {
        return HtmlComponentFactory.inputPassword(id);
    }

    public static Button button(String id) {
        return HtmlComponentFactory.button(id);
    }

    public static Link link(String id) {
        return HtmlComponentFactory.a(id);
    }

    public static CheckBox checkbox(String id) {
        return HtmlComponentFactory.checkbox(id);
    }

    public static Radio radio(String id) {
        return HtmlComponentFactory.radio(id);
    }

    public static DropDown dropdown(String id) {
        return HtmlComponentFactory.dropdown(id);
    }

    public static AlertBox alertbox() {
        return HtmlComponentFactory.alertbox();
    }

    public static FileDialog fileDialog(String id) {
        return HtmlComponentFactory.file(id);
    }

    public static Panel panel(String id) {
        return HtmlComponentFactory.div(id);
    }

    public static Image image(String id) {
        return HtmlComponentFactory.img(id);
    }

    public static DataGrid datagrid(String id) {
        return HtmlComponentFactory.datagrid(id);
    }

    public static ComboBox combobox(String id) {
        throw new ComponentException("There is no combobox concept in Html");
    }

    public static org.testatoo.cartridge.html4.component.ListBox findListBox(By by) {
        return new org.testatoo.cartridge.html4.component.ListBox(evaluator(), findComponent(by).id());
    }

    public static List<org.testatoo.cartridge.html4.component.ListBox> findListBoxes(By by) {
        List<org.testatoo.cartridge.html4.component.ListBox> listBoxes = new ArrayList<org.testatoo.cartridge.html4.component.ListBox>();
        for (Component component : findComponents(by)) {
            listBoxes.add(new org.testatoo.cartridge.html4.component.ListBox(evaluator(), component.id()));
        }
        return listBoxes;
    }

    private static Component findComponent(By by) {
        Component component;
        try {
            component = by.findComponent(evaluator());
        } catch (Exception e) {
            throw new ComponentException("Cannot find component defined " + by, e);
        }
        return component;
    }

    private static List<Component> findComponents(By by) {
        List<Component> component = new ArrayList<Component>();
        try {
            component.addAll(by.findComponents(evaluator()));
        } catch (Exception e) {
            throw new ComponentException("Cannot find component defined " + by, e);
        }
        return component;
    }

    private static HtmlEvaluator evaluator() {
        return EvaluatorHolder.get();
    }

}
